import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { History } from 'history'
import Toast from '../component/toast/index'
import Pop2Setting from '../component/pop2'
import { kToastDurtion, kPasswordLen } from '../../config/index'
import { LanguageEnum } from '../../models/Enum'
import { IStoreState } from '../../reducer/index'
import * as KeyChainActions from '../../actions/keychain'
import * as SettingActions from '../../actions/setting'
import StorageHelper from '../../utils/StorageHelper'
import LocalizationHelper from '../../localization/LocalizationHelper'

import DMCScatter from '../../models/DMCScatter'
import RandomFactory from '../../utils/RandomFactory'
import EncryptHelper from '../../utils/EncryptHelper'
import DMCScatterInterface from '../../interface/Interface'
import PluginFactory from '../../plugins/PluginFactory'
import { passwordRegex } from '../../utils/regex'

interface IEntryProps {
    locked: boolean
    password: string
    language: LanguageEnum
    dispatch?: Dispatch
    history?: History
}

interface IEntryState {
    password: string
    passwordRepeat: string
}

class Entry extends Component<IEntryProps, IEntryState> {
    private keyChainAction: typeof KeyChainActions
    private settingAction: typeof SettingActions
    private popSetting: Pop2Setting

    constructor(props: IEntryProps) {
        super(props)
        this.keyChainAction = bindActionCreators(
            KeyChainActions,
            props.dispatch
        )
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
        this.state = {
            password: '',
            passwordRepeat: '',
        }
    }

    async componentDidMount() {
        const scatter = await StorageHelper.getScatter()
        LocalizationHelper.setLan(scatter.setting.language)
        this.keyChainAction.initKV({
            keypairs: scatter.keyChain.keypairs,
            identitys: scatter.keyChain.identitys,
            permissions: scatter.keyChain.permissions,
        })
        this.settingAction.initKV({
            locked: scatter.setting.locked,
            password: scatter.setting.password,
            networks: scatter.setting.networks,
            language: scatter.setting.language,
            created: scatter.setting.created,
        })
        if (scatter.setting.password.length > 0) {
            const { history } = this.props
            history.replace('/main')
        }
    }

    /**
     * 创建钱包
     */
    createWallet() {
        const { password } = this.state
        const { history } = this.props
        Toast.loading(LocalizationHelper.valueForKey('create_wallet_ing'), 0)
        const hashed = EncryptHelper.md5(
            EncryptHelper.md5(password),
            RandomFactory.systemSalt()
        )
        const networks = PluginFactory.signatureProviders().map((plugin) =>
            plugin.getEndorsedNetwork().clone()
        )

        const scatter = new DMCScatter()
        scatter.setting.password = hashed
        scatter.setting.networks = networks

        DMCScatterInterface.saveScatter(scatter)
        this.settingAction.updateKV('password', hashed)
        this.settingAction.updateKV('networks', networks)
        Toast.hide()
        history.replace('/main')
    }

    /**
     * 检测创建钱包的参数
     */
    prepareCreateWallet() {
        const { password, passwordRepeat } = this.state
        if (password.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return
        }
        if (passwordRepeat.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('repeat_password_empty'),
                kToastDurtion
            )
            return
        }
        if (password !== passwordRepeat) {
            Toast.info(
                LocalizationHelper.valueForKey('password_different'),
                kToastDurtion
            )
            return
        }
        if (password.length < kPasswordLen) {
            Toast.info(
                LocalizationHelper.valueForKey('password_shortest_len') +
                    kPasswordLen,
                kToastDurtion
            )
            return
        }
        if (!passwordRegex.test(password)) {
            Toast.info(
                LocalizationHelper.valueForKey('password_rule'),
                kToastDurtion
            )
            return
        }
        this.createWallet()
    }

    /**
     * 解锁钱包
     */
    unlockWallet() {
        const { password } = this.state
        if (password.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return
        }
        if (password.length < kPasswordLen) {
            Toast.info(
                LocalizationHelper.valueForKey('password_shortest_len') +
                    kPasswordLen,
                kToastDurtion
            )
            return
        }
        const unlocked = DMCScatterInterface.unlockScatter(password)
        if (unlocked) {
            this.settingAction.updateKV('locked', false)
            const { history } = this.props
            history.replace('/main')
        } else {
            Toast.fail(
                LocalizationHelper.valueForKey('password_error'),
                kToastDurtion
            )
        }
    }

    mouseEnterSetting(evt: React.MouseEvent) {
        const target = evt.target as HTMLElement
        const offsetY = target.parentElement.parentElement.scrollTop
        const isChild = evt.target !== evt.currentTarget
        let y: number = isChild
            ? target.parentElement.offsetTop
            : target.offsetTop
        this.popSetting.show({
            r: 10,
            y: y + 44 - offsetY,
            options: [
                {
                    title: LocalizationHelper.valueForKey('language'),
                },
                // {
                //     title: LocalizationHelper.valueForKey("chain_manager"),
                // },
            ],
        })
    }

    onSettingSelect(idx: number) {
        const { history } = this.props
        switch (idx) {
            case 0:
                history.push(`/language`)
                break
            case 1:
                history.push(`/chain`)
            default:
                break
        }
    }

    renderCreate() {
        const { password, passwordRepeat } = this.state
        return (
            <div className='field'>
                <input
                    className='input'
                    type='password'
                    title={LocalizationHelper.valueForKey('password_rule')}
                    placeholder={LocalizationHelper.valueForKey(
                        'password_rule'
                    )}
                    value={password}
                    onChange={(e) =>
                        this.setState({ password: e.target.value.trim() })
                    }
                />
                <input
                    title={LocalizationHelper.valueForKey('password_rule')}
                    className='input repeat'
                    type='password'
                    placeholder={LocalizationHelper.valueForKey(
                        'password_rule'
                    )}
                    value={passwordRepeat}
                    onChange={(e) =>
                        this.setState({ passwordRepeat: e.target.value.trim() })
                    }
                />
                <div
                    className='button'
                    onClick={() => this.prepareCreateWallet()}
                >
                    {LocalizationHelper.valueForKey('create_wallet')}
                </div>
            </div>
        )
    }

    renderUnlock() {
        const { password } = this.state
        return (
            <div className='field'>
                <input
                    title={LocalizationHelper.valueForKey(
                        'enter_plugin_password'
                    )}
                    className='input'
                    type='password'
                    placeholder={LocalizationHelper.valueForKey(
                        'enter_plugin_password'
                    )}
                    value={password}
                    onChange={(e) =>
                        this.setState({ password: e.target.value.trim() })
                    }
                />
                <div className='button' onClick={() => this.unlockWallet()}>
                    {LocalizationHelper.valueForKey('unlock_wallet')}
                </div>
            </div>
        )
    }

    render() {
        const { locked } = this.props
        return (
            <div className='entry'>
                {/* <div className="menu_icon"> */}
                <span className='menu_icon'>
                    <img
                        className='back_icon'
                        width={33}
                        height={30}
                        src={require('../../assets/outside_setting.png')}
                        onClick={(evt) => this.mouseEnterSetting(evt)}
                    />
                </span>
                {/* </div> */}
                <img className='logo' src={require('../../assets/logo.png')} />
                {locked ? this.renderUnlock() : this.renderCreate()}
                <Pop2Setting
                    ref={(e) => (this.popSetting = e)}
                    callback={(idx) => this.onSettingSelect(idx)}
                />
            </div>
        )
    }
}

function mapStateToProps({ setting }: IStoreState) {
    return {
        locked: setting.locked,
        password: setting.password,
        language: setting.language,
    }
}

export default connect(mapStateToProps)(Entry)
