import React, { Component } from 'react'
import { bindActionCreators, Dispatch } from 'redux'
import { connect } from 'react-redux'
import { History } from 'history'
import Father from '../component/father'
import LocalizationHelper from '../../localization/LocalizationHelper'
import { IStoreState } from '../../reducer'
import Network from '../../models/Network'
import * as CreateActions from '../../actions/create'
import * as SettingActions from '../../actions/setting'
import PluginFactory from '../../plugins/PluginFactory'
import Toast from '../component/toast/index'
import {
    kToastDurtion,
    requiredByte,
    requiredCpu,
    requiredNet,
} from '../../config'
import DMCScatterInterface from '../../interface/Interface'
import Plugin from '../../plugins/Plugin'
import Keypair from '../../models/Keypair'
import RandomFactory from '../../utils/RandomFactory'
import EncryptHelper from '../../utils/EncryptHelper'
import { chain_host, version, chain_port, chain_chainId } from '../../config'
import DMC from 'dmc.js'
import reformChainError from '../../utils/reformError'
import BigNumber from 'bignumber.js'

interface ICreateProps {
    history?: any
    networks: Array<Network>
    dispatch?: Dispatch
    name: string
    network: number
    privateKey: string
    created: boolean
    identitys?: any
    keypairs?: any
}

class BackUp extends Component<ICreateProps> {
    private createAction: typeof CreateActions
    private settingAction: typeof SettingActions

    constructor(props: ICreateProps) {
        super(props)
        this.createAction = bindActionCreators(CreateActions, props.dispatch)
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
    }

    async componentDidMount() {
        const { history } = this.props
        if (history.location.state.status === 'costToCreate') {
            return
        }
        this.createAction.updateKV(
            'privateKey',
            LocalizationHelper.valueForKey('generate_privatekey_ing')
        )
        const { network, networks } = this.props
        const network_ = networks[network]
        const plugin = PluginFactory.basePlugin()
        const privateKey = await plugin.randomPrivateKey()
        this.createAction.updateKV('privateKey', privateKey)
    }

    /**
     * 创建账户
     */
    async createDidClick() {
        const { network, networks, name, privateKey, history } = this.props
        if (privateKey.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('generate_privatekey'),
                kToastDurtion
            )
            return
        }
        Toast.loading(LocalizationHelper.valueForKey('create_account_ing'), 0)
        const network_ = networks[network]
        const plugin = await PluginFactory.plugin(network_.blockchain)
        const publicKey = plugin.privateToPublic(privateKey)
        plugin
            .createAccountBySystem(name, publicKey)
            .then(() => {
                Toast.success(
                    LocalizationHelper.valueForKey('create_account_success'),
                    kToastDurtion
                )
                DMCScatterInterface.markHasCreated()
                this.settingAction.updateKV('created', true)
                this.checkAccounts(plugin, network_, publicKey, privateKey)
            })
            .catch((err) => {
                if (err && err?.code === '01') {
                    Toast.fail(
                        LocalizationHelper.valueForKey(
                            'regist_after_hour'
                        ).replace(
                            'TIMESTAMP',
                            Number(
                                new BigNumber(err?.error?.age || 3600000).div(
                                    1000
                                )
                            ).toString()
                        ),
                        kToastDurtion
                    )
                    return
                }
                Toast.hide()
                Toast.fail(
                    LocalizationHelper.valueForKey('create_account_failed'),
                    kToastDurtion
                )
            })
    }

    async costTocreateDidClick() {
        const { history, identitys, keypairs, network, networks } = this.props
        const { hash, create_name, fromAccount } = history?.location?.state
        const newPublicKey = history?.location?.state.publicKey
        const newPrivateKey = history?.location?.state.privateKey
        Toast.loading(LocalizationHelper.valueForKey('create_account_ing'), 0)
        const identity = identitys.find(
            (identity: any) => identity.hash === hash
        )
        const keypair = keypairs.find(
            (keypair: any) => keypair.publicKey === identity.publicKey
        )
        const privateKey = EncryptHelper.aesDecrypt(
            keypair.iv,
            RandomFactory.systemSalt(),
            keypair.privateKey
        )
        const network_ = networks[network]
        const plugin2 = await PluginFactory.plugin(network_.blockchain)
        const publicKey = plugin2.privateToPublic(privateKey)
        const plugin = await PluginFactory.plugin(network_.blockchain)
        const dmc = DMC({
            chainId: chain_chainId,
            keyProvider: privateKey,
            httpEndpoint: `http://${chain_host}:${chain_port}`,
            logger: {
                log: null,
                error: null,
            },
            keyPrefix: 'DM',
        })
        const actions = [
            {
                account: 'dmc',
                name: 'newaccount',
                authorization: [
                    {
                        actor: fromAccount,
                        permission: 'active',
                    },
                ],
                data: {
                    creator: fromAccount,
                    name: create_name,
                    owner: {
                        threshold: 1,
                        keys: [
                            {
                                key: newPublicKey,
                                weight: 1,
                            },
                        ],
                        accounts: [] as any,
                        waits: [] as any,
                    },
                    active: {
                        threshold: 1,
                        keys: [
                            {
                                key: newPublicKey,
                                weight: 1,
                            },
                        ],
                        accounts: [] as any,
                        waits: [] as any,
                    },
                },
            },
            {
                account: 'dmc',
                name: 'buyrambytes',
                authorization: [
                    {
                        actor: fromAccount,
                        permission: 'active',
                    },
                ],
                data: {
                    payer: fromAccount,
                    receiver: create_name,
                    bytes: requiredByte,
                },
            },
            {
                account: 'dmc',
                name: 'delegatebw',
                authorization: [
                    {
                        actor: fromAccount,
                        permission: 'active',
                    },
                ],
                data: {
                    from: fromAccount,
                    receiver: create_name,
                    stake_net_quantity: `${requiredCpu} DMC`,
                    stake_cpu_quantity: `${requiredNet} DMC`,
                    transfer: 1,
                },
            },
        ]
        dmc.transact(
            { actions },
            {
                blocksBehind: 3,
                expireSeconds: 30,
            }
        )
            .then((resp: any) => {
                Toast.hide()
                Toast.success(
                    LocalizationHelper.valueForKey('create_account_success'),
                    kToastDurtion
                )
                this.checkAccounts(
                    plugin,
                    network_,
                    newPublicKey,
                    newPrivateKey
                )
                history.push('/main')
            })
            .catch((error: any) => {
                Toast.hide()
                reformChainError(error)
            })
    }

    checkAccounts(
        plugin: Plugin,
        network: Network,
        publicKey: string,
        privateKey: string
    ) {
        Toast.loading(LocalizationHelper.valueForKey('import_account_ing'), 0)
        setTimeout(async () => {
            const iv = RandomFactory.iv()
            let kp = new Keypair({
                blockChain: network.blockchain,
                name: RandomFactory.string(8, true),
                publicKey: publicKey,
                privateKey: EncryptHelper.aesEncrypt(
                    iv,
                    RandomFactory.systemSalt(),
                    privateKey
                ),
                iv,
            })
            const accounts = await plugin.importAccount(kp, network)
            if (accounts.toString() === 'TypeError: Failed to fetch') {
                reformChainError(accounts)
                return
            }
            if (accounts.length <= 0) {
                this.checkAccounts(plugin, network, publicKey, privateKey)
            } else {
                Toast.hide()
                const { history } = this.props
                this.createAction.clear()
                history.replace('/selectaccount', {
                    accounts,
                    network,
                    kp,
                })
            }
        }, 2000)
    }

    render() {
        const { history, privateKey, created } = this.props
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('create_accout')}
            >
                <div className='import'>
                    <span className='tag'>
                        {LocalizationHelper.valueForKey('backup_privatekey')}
                    </span>
                    <div className='textarea scroll pk'>{privateKey}</div>
                    <div
                        className='continue'
                        onClick={() =>
                            created
                                ? this.costTocreateDidClick()
                                : this.createDidClick()
                        }
                    >
                        {LocalizationHelper.valueForKey('create')}
                    </div>
                </div>
            </Father>
        )
    }
}

function mapStateToProps({ setting, create, keychain }: IStoreState) {
    return {
        networks: setting.networks,
        network: create.network,
        name: create.name,
        privateKey: create.privateKey,
        created: setting.created,
        keypairs: keychain.keypairs,
        identitys: keychain.identitys,
    }
}

export default connect(mapStateToProps)(BackUp)
