import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { match } from 'react-router-dom'
import Father from '../component/father'
import { History } from 'history'
import LocalizationHelper from '../../localization/LocalizationHelper'
import { IStoreState } from '../../reducer'
import * as SettingActions from '../../actions/setting'
import { LanguageEnum } from '../../models/Enum'
import Toast from '../component/toast/index'
import { kPasswordLen, kToastDurtion } from '../../config'
import EncryptHelper from '../../utils/EncryptHelper'
import RandomFactory from '../../utils/RandomFactory'
import DMCScatterInterface from '../../interface/Interface'
import { chain_host, version } from '../../config'
import PluginFactory from '../../plugins/PluginFactory'
import DMCScatter from '../../models/DMCScatter'
import { passwordRegex } from '../../utils/regex'

interface ILanguageProps {
    match?: match
    history?: History
    dispatch?: Dispatch
    language: LanguageEnum
    password: string
}

class ChangePassword extends Component<ILanguageProps> {
    private settingAction: typeof SettingActions

    constructor(props: ILanguageProps) {
        super(props)
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
    }

    state = {
        present_password: '',
        new_password: '',
        confirm_password: '',
    }

    checkPwd(pwd: string): boolean {
        const { password } = this.props
        if (pwd.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return false
        }
        const hashed = EncryptHelper.md5(
            EncryptHelper.md5(pwd),
            RandomFactory.systemSalt()
        )
        if (hashed !== password) {
            Toast.info(
                LocalizationHelper.valueForKey('password_error'),
                kToastDurtion
            )
            return false
        }
        return true
    }

    changePassword = () => {
        const { history } = this.props
        const { present_password, new_password } = this.state
        const hashed = EncryptHelper.md5(
            EncryptHelper.md5(new_password),
            RandomFactory.systemSalt()
        )
        if (
            !DMCScatterInterface.updatePassword(present_password, new_password)
        ) {
            return
        }
        this.settingAction.updateKV('password', hashed)
        history.replace('/main')
        Toast.info(LocalizationHelper.valueForKey('success'), kToastDurtion)
    }

    confirmChange = () => {
        const { present_password, new_password, confirm_password } = this.state
        const pass = this.checkPwd(present_password)
        if (!pass) {
            return
        }
        if (new_password.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('password_empty'),
                kToastDurtion
            )
            return
        }
        if (confirm_password.length <= 0) {
            Toast.info(
                LocalizationHelper.valueForKey('repeat_password_empty'),
                kToastDurtion
            )
            return
        }
        if (new_password !== confirm_password) {
            Toast.info(
                LocalizationHelper.valueForKey('password_different'),
                kToastDurtion
            )
            return
        }
        if (new_password.length < kPasswordLen) {
            Toast.info(
                LocalizationHelper.valueForKey('password_shortest_len') +
                    kPasswordLen,
                kToastDurtion
            )
            return
        }
        if (!passwordRegex.test(new_password)) {
            Toast.info(
                LocalizationHelper.valueForKey('password_rule'),
                kToastDurtion
            )
            return
        }
        this.changePassword()
    }

    render() {
        const { history } = this.props
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('change_password')}
            >
                <div className='verify changePassword'>
                    <span className='input_tips' style={{ marginTop: 12 }}>
                        {LocalizationHelper.valueForKey('password_tip')}
                    </span>
                    <div className='presentPassword'>
                        <span>
                            {LocalizationHelper.valueForKey('present_password')}
                        </span>
                        <input
                            title={LocalizationHelper.valueForKey(
                                'input_current_password'
                            )}
                            className='textarea scroll input'
                            placeholder={LocalizationHelper.valueForKey(
                                'input_current_password'
                            )}
                            type='password'
                            value={this.state.present_password}
                            onChange={(e) =>
                                this.setState({
                                    present_password: e.target.value,
                                })
                            }
                        />
                    </div>
                    <div className='newPassword'>
                        <span>
                            {LocalizationHelper.valueForKey('set_new_password')}
                        </span>
                        <input
                            title={LocalizationHelper.valueForKey(
                                'password_rule'
                            )}
                            className='textarea scroll input'
                            placeholder={LocalizationHelper.valueForKey(
                                'password_rule'
                            )}
                            type='password'
                            value={this.state.new_password}
                            onChange={(e) =>
                                this.setState({
                                    new_password: e.target.value,
                                })
                            }
                        />
                    </div>
                    <div className='confirmPassword'>
                        <span>
                            {LocalizationHelper.valueForKey(
                                'confirm_new_password'
                            )}
                        </span>
                        <input
                            title={LocalizationHelper.valueForKey(
                                'confirm_password_placeHolder'
                            )}
                            className='textarea scroll input'
                            placeholder={LocalizationHelper.valueForKey(
                                'confirm_password_placeHolder'
                            )}
                            type='password'
                            value={this.state.confirm_password}
                            onChange={(e) =>
                                this.setState({
                                    confirm_password: e.target.value,
                                })
                            }
                        />
                    </div>
                    <div
                        className='btn'
                        onClick={() => {
                            this.confirmChange()
                        }}
                    >
                        {LocalizationHelper.valueForKey('change')}
                    </div>
                </div>
            </Father>
        )
    }
}

function mapStateToProps({ setting }: IStoreState) {
    return {
        language: setting.language,
        password: setting.password,
    }
}

export default connect(mapStateToProps)(ChangePassword)
