import React, { Component } from 'react'
import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'
import { match } from 'react-router-dom'
import Father from '../component/father'
import { History } from 'history'
import LocalizationHelper from '../../localization/LocalizationHelper'
import { IStoreState } from '../../reducer'
import * as SettingActions from '../../actions/setting'
import { LanguageEnum } from '../../models/Enum'
import Toast from '../component/toast/index'
import { kPasswordLen, kToastDurtion } from '../../config'
import EncryptHelper from '../../utils/EncryptHelper'
import RandomFactory from '../../utils/RandomFactory'
import DMCScatterInterface from '../../interface/Interface'
import { chain_host, version, chain_port, chain_chainId } from '../../config'
import PluginFactory from '../../plugins/PluginFactory'
import DMCScatter from '../../models/DMCScatter'
import DMC from 'dmc.js'
import { amountRegex } from '../../utils/regex'
import ConfirmModal from '../component/confirm-modal'

interface ILanguageProps {
    match?: match
    history?: any
    dispatch?: Dispatch
    language: LanguageEnum
    password: string
    identitys?: any
    keypairs?: any
}

class ChangePassword extends Component<ILanguageProps> {
    private settingAction: typeof SettingActions
    private confirmModal: ConfirmModal

    constructor(props: ILanguageProps) {
        super(props)
        this.settingAction = bindActionCreators(SettingActions, props.dispatch)
    }

    state = {
        unlock_number: '',
    }

    checkNumberPrecision = (number: string, precision: number) => {
        const numArr = number.split('.')
        if (numArr.length === 1) {
            return true
        }
        if (numArr.length === 2) {
            const decimal = numArr[1]
            if (decimal.length > precision) {
                return false
            }
            return true
        }
        return false
    }

    clickUnlock = (evt: any) => {
        const { unlock_number } = this.state
        if (unlock_number === '') {
            Toast.info(
                LocalizationHelper.valueForKey('unlock_amount_input'),
                kToastDurtion
            )
            return
        }
        if (!amountRegex.test(unlock_number)) {
            Toast.info(
                LocalizationHelper.valueForKey('invaild_quality'),
                kToastDurtion
            )
            return
        }
        if (!this.checkNumberPrecision(unlock_number, 4)) {
            Toast.info(
                LocalizationHelper.valueForKey('precision_error'),
                kToastDurtion
            )
            return
        }
        const target = evt.target as HTMLElement
        const offsetY = target.parentElement.parentElement.scrollTop
        let y: number = target.offsetTop
        this.confirmModal.show({
            y: 44 + 120,
        })
    }

    confirmUnlock = () => {
        Toast.loading('', 0)
        const { history, identitys, keypairs } = this.props
        const { unlock_number } = this.state
        const { hash, locked_amount, locked_time, present_account } =
            history?.location?.state
        const identity = identitys.find(
            (identity: any) => identity.hash === hash
        )
        const name = identity.name.split('@')[0]
        const keypair = keypairs.find(
            (keypair: any) => keypair.publicKey === identity.publicKey
        )
        const privateKey = EncryptHelper.aesDecrypt(
            keypair.iv,
            RandomFactory.systemSalt(),
            keypair.privateKey
        )
        const dmc = DMC({
            chainId: chain_chainId,
            keyProvider: privateKey,
            httpEndpoint: `http://${chain_host}:${chain_port}`,
            logger: {
                log: null,
                error: null,
            },
            keyPrefix: 'DM',
        })
        const actions = [
            {
                account: 'dmc.token',
                name: 'exunlock',
                authorization: [
                    {
                        actor: name,
                        permission: 'active',
                    },
                ],
                data: {
                    owner: name,
                    quantity: {
                        quantity: `${Number(unlock_number).toFixed(4)} DMC`,
                        contract: 'datamall',
                    },
                    expiration: locked_time,
                    memo: '',
                },
            },
        ]
        dmc.transact(
            { actions },
            {
                blocksBehind: 3,
                expireSeconds: 30,
            }
        )
            .then((resp: any) => {
                Toast.hide()
                Toast.success(
                    LocalizationHelper.valueForKey('unlock_success'),
                    kToastDurtion
                )
                setTimeout(() => {
                    history.push('/lock', {
                        present_account: present_account,
                        hash: hash,
                    })
                }, kToastDurtion * 1000)
                // this.setState({
                //     unlock_number: '',
                // })
            })
            .catch((error: any) => {
                Toast.hide()
                Toast.fail(
                    LocalizationHelper.valueForKey('unlock_failed'),
                    kToastDurtion
                )
            })
    }

    render() {
        const { history, password } = this.props
        const { locked_amount, locked_time } = history?.location?.state
        const btnDisabled = new Date(`${locked_time}Z`).getTime() > Date.now()
        return (
            <Father
                history={history}
                left={require('../../assets/back.png')}
                title={LocalizationHelper.valueForKey('extransfer')}
            >
                <div className='verify extransfer'>
                    <div className='amount'>
                        <div className='amount_top'>
                            <span>
                                {LocalizationHelper.valueForKey(
                                    'unlock_amount'
                                )}
                            </span>
                        </div>
                        <div className='amount_bottom'>
                            <input
                                className='textarea scroll input'
                                placeholder={locked_amount}
                                value={this.state.unlock_number}
                                onChange={(e) =>
                                    this.setState({
                                        unlock_number: e.target.value,
                                    })
                                }
                            />
                            <div
                                className='transfer_all'
                                onClick={() => {
                                    this.setState({
                                        unlock_number: locked_amount,
                                    })
                                }}
                            >
                                {LocalizationHelper.valueForKey('all_unlock')}
                            </div>
                        </div>
                    </div>
                    <div
                        className={`btn ${btnDisabled ? 'disabledBtn' : ''}`}
                        onClick={(evt) => {
                            if (btnDisabled) return
                            this.clickUnlock(evt)
                        }}
                    >
                        {LocalizationHelper.valueForKey('ex_unlock')}
                    </div>
                </div>
                <ConfirmModal
                    ref={(e) => (this.confirmModal = e)}
                    callback={this.confirmUnlock}
                    password={password}
                ></ConfirmModal>
            </Father>
        )
    }
}

function mapStateToProps({ setting, keychain }: IStoreState) {
    return {
        language: setting.language,
        password: setting.password,
        keypairs: keychain.keypairs,
        identitys: keychain.identitys,
    }
}

export default connect(mapStateToProps)(ChangePassword)
